string description = "jl: simple .fx demo with bump reflection";

float4x4 WIT : WorldInverseTranspose;
float4x4 WVP : WorldViewProjection;
float4x4 W : World;
float4x4 VI : ViewInverse;

float BumpFactor
<
	string UIWidget = "Slider";
	float UIMin = 0.0;
	float UIMax = 1.0;
	float UIStep = 0.01;
	string UIName = "Bump Intensity";
> = 0.3;

texture EnviTexture : Environment
<
	string ResourceName = "nvlobby_cube_mipmap.dds";
	string TextureType = "Cube";	
>;

sampler EnviMap = sampler_state 
{
	texture = <EnviTexture>;
	AddressU  = CLAMP;
	AddressV  = CLAMP;
	AddressW  = CLAMP;
	MIPFILTER = LINEAR;
	MINFILTER = LINEAR;
	MAGFILTER = LINEAR;
};

texture NormalTexture
<
	string ResourceName = "Default_bump_normal.dds";
    string TextureType = "2D";	
>;

sampler NormalMap = sampler_state 
{
	texture = <NormalTexture>;
	AddressU  = CLAMP;
	AddressV  = CLAMP;
	AddressW  = CLAMP;
	MIPFILTER = LINEAR;
	MINFILTER = LINEAR;
	MAGFILTER = LINEAR;
};

struct appdata
{
	float3 Position	: POSITION;
	float3 Normal	: NORMAL;
	float3 Tangent	: TANGENT;
	float3 Binormal	: BINORMAL;
	float2 UV		: TEXCOORD0;
};

struct vertexOutput
{
	float4 HPosition	: POSITION;
	float2 UV			: TEXCOORD0;
	float3 Normal		: TEXCOORD1;
	float3 Tangent		: TEXCOORD2;
	float3 Binormal		: TEXCOORD3;
	float3 View			: TEXCOORD4;	
};

struct pixelOutput
{
	float4 col : COLOR;
};

vertexOutput myVertexShader(appdata IN)
{
	vertexOutput OUT;
	float4 Po = float4(IN.Position, 1.0);	
	OUT.HPosition = mul(Po, WVP);
	OUT.Normal = normalize(mul(float4(IN.Normal, 0.0), WIT).xyz);
	OUT.Tangent = normalize(mul(float4(IN.Tangent, 0.0), W).xyz);
	OUT.Binormal = normalize(mul(float4(IN.Binormal, 0.0), W).xyz);
	float4 Pw = mul(Po, W);
	OUT.View = normalize(Pw.xyz-VI[3].xyz); // zeigt vom Viewer her
	OUT.UV = IN.UV;
	return OUT;
}

pixelOutput myPixelShader(vertexOutput IN)
{
	pixelOutput OUT; 
	float3 normal = normalize(IN.Normal);
	float3 tangent = normalize(IN.Tangent);
	float3 binormal = normalize(IN.Binormal);
	float3 bumps = 2.0*tex2D(NormalMap, IN.UV).xyz-1.0;
	float3 bumpNorm = normalize((1-BumpFactor)*normal+BumpFactor*(bumps.x*tangent+bumps.y*binormal+bumps.z*normal));
	OUT.col = texCUBE(EnviMap, reflect(IN.View, bumpNorm));
	return OUT;
}

technique dx9
{
	pass p0 
	{		
		VertexShader = compile vs_2_0 myVertexShader();
		ZEnable = true;
		ZWriteEnable = true;
		CullMode = None;
		PixelShader = compile ps_2_0 myPixelShader();
	}
}